Package: BiocBuildReporter
Type: Package
Title: Functions to process a bioconductor build report database
Version: 1.1.1
Description: This package reads remote parquet files that have processed Bioconductor build report logs. Users may query the tables directly for specific information or use pre-defined helper functions for common queries. The logs processed are from https://bioconductor.org/checkResults/. In the future we will extend this package out to include processing of r-universe logs.
Depends: R (>= 4.6.0)
Imports: arrow, dplyr, BiocFileCache
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Authors@R: c(
    person(given="Sean", family="Davis", role = c("aut"), email = 'seandavi@gmail.com'),
    person(given="Lori", family="Shepherd", role = c("aut", "cre"), email = "lori.shepherd@roswellpark.org",comment=c(ORCID = "0000-0002-3242-0582")))    
Suggests: BiocStyle, testthat (>= 3.0.0), knitr, rmarkdown, ggplot2,
        tidyr, stringr
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/lshep/BiocBuildReporter.git
BugReports: https://github.com/lshep/BiocBuildReporter/issues
biocViews: Software, Infrastructure
git_url: https://git.bioconductor.org/packages/BiocBuildReporter
git_branch: devel
git_last_commit: 13ae3c4
git_last_commit_date: 2026-05-20
Repository: Bioconductor 3.24
Date/Publication: 2026-06-01
NeedsCompilation: no
Packaged: 2026-06-01 21:31:56 UTC; biocbuild
Author: Sean Davis [aut],
  Lori Shepherd [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>)
Maintainer: Lori Shepherd <lori.shepherd@roswellpark.org>

# Generated by roxygen2: do not edit by hand

export(get_all_bbs_tables)
export(get_bbs_table)
export(get_build_report)
export(get_failing_packages)
export(get_latest_branches)
export(get_package_build_results)
export(get_package_release_info)
export(list_all_branches)
export(list_all_builders)
export(package_error_count)
export(package_failures_over_time)
import(BiocFileCache)
import(arrow)
import(dplyr)

# BiocBuildReporter


The BiocBuildReporter package provides access to years of Bioconductor build
system data, representing a comprehensive record of package builds across:

- **Thousands of packages** in the Bioconductor ecosystem
- **Multiple R versions** spanning several years of development
- **Multiple platforms** including Linux, macOS, and Windows
- **Different build stages** (install, build, check)

The Bioconductor build system runs regularly, testing all packages to ensure
they meet quality standards and work correctly across different platforms. This
dataset captures the results of these builds, including:

- Build status (OK, WARNING, ERROR, TIMEOUT)
- Package version information
- Git commit information
- Maintainer details
- Propagation status to the community

This package provides functions to explore and analyze this rich dataset to
understand:

- Package build history and stability
- Platform-specific issues
- Package growth over time
- Common failure patterns

Please see vignette for detailed examples.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 6,
  warning = FALSE,
  message = FALSE
)

## ----install, eval = FALSE----------------------------------------------------
# if (!"BiocManager" %in% rownames(installed.packages()))
#      install.packages("BiocManager")
# BiocManager::install("BiocBuildReporter", dependencies=TRUE)

## ----library, results='hide', warning=FALSE, message=FALSE--------------------
library(BiocBuildReporter)

## ----setup--------------------------------------------------------------------
library(BiocBuildReporter)
library(dplyr)
library(ggplot2)
library(tidyr)

## ----get_all_tables-----------------------------------------------------------
# Download all available tables
# This will cache the tables for quick subsequent access
get_all_bbs_tables()

## ----get_individual_table-----------------------------------------------------
# Get the build summary table
build_summary <- get_bbs_table("build_summary")

# Get the info table
info <- get_bbs_table("info")

# Get the propagation status table
propagation_status <- get_bbs_table("propagation_status")

## ----read_remote--------------------------------------------------------------
info <- get_bbs_table("info", useLocal=FALSE)

## ----updateTables-------------------------------------------------------------
info <- get_bbs_table("info", useLocal=TRUE, updateLocal=TRUE)

## ----package_release_info-----------------------------------------------------
# Get release information for BiocFileCache
bfc_releases <- get_package_release_info("BiocFileCache")
bfc_releases

## ----get_package_build_results------------------------------------------------
# Get build results for BiocFileCache on branch RELEASE_3_22
get_package_build_results("BiocFileCache", branch="RELEASE_3_22")

## ----package_error_count------------------------------------------------------
# Get error counts for BiocFileCache
bfc_errors <- package_error_count("BiocFileCache")
bfc_errors

# Filter to a specific branch
bfc_errors_release <- package_error_count("BiocFileCache", branch = "RELEASE_3_22")
bfc_errors_release

# Filter to a specific builder
bfc_errors_builder <- package_error_count("BiocFileCache", 
                                          builder = "nebbiolo2", 
                                          branch = "RELEASE_3_22")
bfc_errors_builder

## ----filter_devel_errors------------------------------------------------------
# Get devel errors
dev_errors <- package_error_count("BiocFileCache", branch = "devel")

# Filter to current devel version
dev_errors |> filter(version == max(version))

## ----package_failure_over_time------------------------------------------------
# Get failure events for BiocFileCache on nebbiolo1 and
# group events in a 24 hour period
package_failures_over_time("BiocFileCache", "nebbiolo1", 24)

## ----package_growth-----------------------------------------------------------
# Get info table
info <- get_bbs_table("info")

# Count unique packages by branch
package_counts <- info |>
  group_by(git_branch) |>
  summarise(
    n_packages = n_distinct(Package),
    .groups = "drop"
  ) |>
  arrange(desc(n_packages))

# Display the counts
package_counts

# Visualize package counts by branch
ggplot(package_counts, aes(x = reorder(git_branch, n_packages), y = n_packages)) +
  geom_col(fill = "steelblue") +
  coord_flip() +
  labs(
    title = "Number of Packages by Bioconductor Branch",
    x = "Branch",
    y = "Number of Packages"
  ) +
  theme_minimal()

## ----build_status-------------------------------------------------------------
# Get build summary table
build_summary <- get_bbs_table("build_summary")

# Count build statuses
status_counts <- build_summary |>
  count(status) |>
  arrange(desc(n))

status_counts

# Visualize status distribution
ggplot(status_counts, aes(x = reorder(status, n), y = n)) +
  geom_col(aes(fill = status)) +
  scale_fill_manual(values = c(
    "OK" = "green3",
    "WARNING" = "orange",
    "ERROR" = "red",
    "TIMEOUT" = "darkred"
  )) +
  coord_flip() +
  labs(
    title = "Distribution of Build Statuses",
    x = "Status",
    y = "Count"
  ) +
  theme_minimal() +
  theme(legend.position = "none")

## ----platform_analysis--------------------------------------------------------
# Analyze build status by platform (node)
platform_status <- build_summary |>
  group_by(node, status) |>
  summarise(count = n(), .groups = "drop") |>
  group_by(node) |>
  mutate(
    total = sum(count),
    percentage = count / total * 100
  ) |>
  ungroup()

# Show error rates by platform
error_rates <- platform_status |>
  filter(status %in% c("ERROR", "TIMEOUT")) |>
  group_by(node) |>
  summarise(
    error_count = sum(count),
    total = first(total),
    error_rate = sum(percentage),
    .groups = "drop"
  ) |>
  arrange(desc(error_rate))

head(error_rates, 10)

## ----stage_analysis-----------------------------------------------------------
# Analyze failures by stage
stage_failures <- build_summary |>
  filter(status %in% c("ERROR", "TIMEOUT")) |>
  count(stage, status) |>
  arrange(desc(n))

stage_failures

# Visualize
ggplot(stage_failures, aes(x = stage, y = n, fill = status)) +
  geom_col() +
  scale_fill_manual(values = c("ERROR" = "red", "TIMEOUT" = "darkred")) +
  labs(
    title = "Build Failures by Stage",
    x = "Build Stage",
    y = "Number of Failures",
    fill = "Status"
  ) +
  theme_minimal()

## ----problematic_packages-----------------------------------------------------
# Find packages with most errors
package_errors <- build_summary |>
  filter(status %in% c("ERROR", "TIMEOUT")) |>
  count(package, status) |>
  group_by(package) |>
  summarise(
    total_errors = sum(n),
    .groups = "drop"
  ) |>
  arrange(desc(total_errors))

# Top 10 packages with most errors
head(package_errors, 10)

## ----maintainer_analysis------------------------------------------------------
# Get unique packages per maintainer
maintainer_packages <- info |>
  group_by(Maintainer) |>
  summarise(
    n_packages = n_distinct(Package),
    packages = paste(unique(Package), collapse = ", "),
    .groups = "drop"
  ) |>
  arrange(desc(n_packages))

# Top maintainers by number of packages
head(maintainer_packages, 10)

# Distribution of packages per maintainer
ggplot(maintainer_packages, aes(x = n_packages)) +
  geom_histogram(binwidth = 1, fill = "steelblue", color = "white") +
  labs(
    title = "Distribution of Packages per Maintainer",
    x = "Number of Packages",
    y = "Number of Maintainers"
  ) +
  theme_minimal()

## ----temporal_analysis--------------------------------------------------------
# Analyze build patterns over time
build_summary <- build_summary |>
  mutate(
    date = as.Date(startedat),
    month = format(startedat, "%Y-%m")
  )

# Build activity by month
monthly_builds <- build_summary |>
  count(month) |>
  mutate(month_date = as.Date(paste0(month, "-01")))

ggplot(monthly_builds, aes(x = month_date, y = n)) +
  geom_line(color = "steelblue", linewidth = 1) +
  geom_point(color = "steelblue") +
  labs(
    title = "Build Activity Over Time",
    x = "Month",
    y = "Number of Builds"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

# Error rate over time
monthly_errors <- build_summary |>
  group_by(month) |>
  summarise(
    total = n(),
    errors = sum(status %in% c("ERROR", "TIMEOUT")),
    error_rate = errors / total * 100,
    .groups = "drop"
  ) |>
  mutate(month_date = as.Date(paste0(month, "-01")))

ggplot(monthly_errors, aes(x = month_date, y = error_rate)) +
  geom_line(color = "red", linewidth = 1) +
  geom_point(color = "red") +
  labs(
    title = "Build Error Rate Over Time",
    x = "Month",
    y = "Error Rate (%)"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

## ----get_build_report---------------------------------------------------------
# Retrieves the build report for all packages on December 29, 2025
# Filtering also for RELEASE_3_22 branch and linux "nebbiolo1" build machine
get_build_report("2025-12-29", branch="RELEASE_3_22", builder="nebbiolo2")

## ----get_failing_packages-----------------------------------------------------
# returns all failing packages for RELEASE_3_22 branch
# for build machine nebbolo2
get_failing_packages("RELEASE_3_22", "nebbiolo2")

## ----sessionInfo--------------------------------------------------------------

sessionInfo()

#' BiocBuildReporter:  Functions to process a bioconductor build report database
#' 
#' The BiocBuildReporter package provides access to years of Bioconductor build
#' system data, representing a comprehensive record of package builds across:
#' - **Thousands of packages** in the Bioconductor ecosystem
#' - **Multiple R versions** spanning several years of development
#' - **Multiple platforms** including Linux, macOS, and Windows
#' - **Different build stages** (install, build, check)
#' 
#'  Please see vignette("BiocBuildReporter") for details on usage and pre-built
#' query functions. 
#'
#' @name BiocBuildReporter
"_PACKAGE"

#' @title Get git information for a given package across all Bioconductor releases
#'
#' @description Get the package version and git commit data for all releases in Bioconductor.
#'
#' @details Utilizes data from the info parquet file to retrieve the equivalent
#' git information for every available Bioconductor release of a package. This
#' include the git commit hash and last commit date along with the package version.
#' 
#' @param packagename a single valid Bioconductor package name
#'
#' @return tibble of release information or NULL if package is not found
#'
#' @aliases get_package_release_info
#'
#' @seealso get_bbs_table
#'
#' @author Lori Shepherd
#'
#' @examples
#' get_package_release_info("BiocFileCache")
#'
#' @import dplyr
#' @export
#' 
get_package_release_info <- function(packagename){

    stopifnot(is.character(packagename), length(packagename)==1L)
    infoTbl <- get_bbs_table("info")
    if(packagename %in% infoTbl$Package){
        infoTbl |>
            filter(Package == packagename) |>
            group_by(git_branch) |>
            slice_max(order_by = git_last_commit_date, n = 1, with_ties = FALSE) |>
            ungroup() |>
            select(Package, Version, git_branch, git_last_commit, git_last_commit_date)
    }else{
        message(sprintf("Package: '%s' Not Found.\n  Please check spelling and capitalization",
                        packagename))
        NULL
    }
}



#' @title Get the most recent build result data for a given package on a given branch
#'
#' @description Get the build status information for a given package on a specified branch.
#'
#' @details Utilizes data from the info and build_summary parquet files to
#' retrieve the latest build results for a specified package on a given
#' branch. If no branch is given it defaults to use 'devel'. The data returned
#' includes the node (builder machine name), stage (install, build, check),
#' package version number, status of stage, date stage completed, and git
#' information for the build (git branch, git commit hash, git commit date).
#' 
#' @param packagename a single valid Bioconductor package name
#' @param branch a bioconductor branch (eg. 'devel', 'RELEASE_3_22')
#' 
#' @return tibble of build status information or NULL if package or branch is not found
#'
#' @aliases get_package_build_results
#'
#' @seealso get_bbs_table
#'
#' @author Lori Shepherd
#'
#' @examples
#' get_package_build_results("BiocFileCache")
#' get_package_build_results("BiocFileCache", branch="RELEASE_3_22")
#'
#' @export
#' 
get_package_build_results <- function(packagename, branch="devel"){

    stopifnot(is.character(packagename), length(packagename)==1L)
    stopifnot(is.character(branch), length(branch)==1L)
    
    summaryTbl <- get_bbs_table("build_summary")
    
    if(!(packagename %in% summaryTbl$package)){
        message(sprintf("Package: '%s' Not Found.\n  Please check spelling and capitalization",
                        packagename))
        return(NULL)
    }
    
    pkgTbl <-
        summaryTbl |> filter(package == packagename) |>
        group_by(node, version, stage) |>
        slice_max(endedat, n = 1, with_ties = FALSE) |>
        ungroup()  |> 
        select(package, node, stage, version, status, endedat)

    infoTbl <- get_bbs_table("info")
    
    if(!(branch %in% infoTbl$git_branch[infoTbl$Package == packagename])){
        message(sprintf("Branch: '%s' Not Found.\n  Please check spelling and capitalization against list_all_branches()",
                        branch))
        return(NULL)
    }
        
    info_latest <-
        infoTbl |> filter(Package == packagename) |>
        group_by(Version) |>
        slice_max(git_last_commit_date, n = 1, with_ties = FALSE) |>
        ungroup() |> 
        select(Version, git_branch, git_last_commit, git_last_commit_date)
    
    if (branch == "devel"){
        info_filtered <-
            info_latest |> filter(git_branch == branch) |>
            mutate(version_obj = package_version(Version)) |>
            slice_max(version_obj, n = 1, with_ties = FALSE) |>
            select(-version_obj)
    }else{
        info_filtered <-
            info_latest |> filter(git_branch == branch)
    }
    
    results <-
        pkgTbl |>
        inner_join(info_filtered, by = c("version" = "Version"))

    return(results)
}

#' @title Get a count of how many time a package failed on the Bioconductor
#' Build System
#'
#' @description Get a count of how many time a package failed (ERROR/TIMEOUT) on the
#' Bioconductor Build System; optionally can specify a specific builder or
#' branch to filter results.
#'
#' @details Utilizes data from the build_summary and info parquet files to
#' retrieve a count of errors (TIMEOUT/ERROR) for  a specified
#' package. Optionally a given builder machine name or Bioconductor branch may
#' be specified to filter request. If no builder is given it will return all. If
#' no branch is give it will also return all. The data returned
#' includes the node (builder machine name), package version number, stage
#' (install, build, check), total number of times that stage/builder tried to
#' run, total number of times the package failed during that stage, and the
#' given Bioconductor git branch.
#' NOTE: since 'devel' is on-going. There will be multiple 'devel' results
#' across multiple years not just the current devel. See example to filtering to
#' see most recent.
#' NOTE: If builder or branch is not found, function will continue and return
#' unfiltered results.
#' 
#' @param packagename a single valid Bioconductor package name
#' @param builder name of a valid Bioconductor build machine (ie 'nebbiolo1')
#' @param branch a bioconductor branch (eg. 'devel', 'RELEASE_3_22')
#' 
#' @return tibble of data including counts of runs and errors or NULL if package
#' is not found
#'
#' @aliases package_error_count
#'
#' @seealso get_bbs_table
#'
#' @author Lori Shepherd
#'
#' @examples
#' package_error_count("BiocFileCache")
#' package_error_count("BiocFileCache", branch="RELEASE_3_22")
#' package_error_count("BiocFileCache", builder = "nebbiolo2", branch="RELEASE_3_22")
#'
#' library(dplyr)
#' ## devel will have more than just current devel
#' devErrors <- package_error_count("BiocFileCache", branch="devel")
#' ## can use package version number to filter to current devel
#' devErrors |> filter(version == max(version))
#' 
#' @export
#' 
package_error_count <- function(packagename, builder=NULL, branch=NULL){

    stopifnot(is.character(packagename), length(packagename) == 1L)
    
    summaryTbl <- get_bbs_table("build_summary")
    
    if(!(packagename %in% summaryTbl$package)){
        message(sprintf("Package: '%s' Not Found.\n  Please check spelling and capitalization",
                        packagename))
        return(NULL)
    }

    pkgTbl <- summaryTbl |> filter(package == packagename)
    
    if (!is.null(builder)){
        if(!builder %in% pkgTbl$node){
            message(sprintf("Builder: '%s' Not Found after filtering for '%s'.\n  Please check spelling and capitalization against list_all_builders()",
                            builder, packagename))
        }else{        
            pkgTbl <- pkgTbl |> filter(node %in% builder)
        }
    }

    countTbl <- pkgTbl |>
        group_by(node, version, stage) |>
        summarise(
            count_total = n(),
            count_error = sum(status %in% c("ERROR", "TIMEOUT")),
            .groups = "drop"
        ) |>
        mutate(
            version = factor(version, levels = as.character(sort(package_version(unique(version)))))
        ) |>
        arrange(version, node)

    infoTbl <- get_bbs_table("info")
    branchTbl <- infoTbl |>
        filter(Package == packagename) |>
        mutate(Version = package_version(Version)) |>
        group_by(Version) |>
        slice_max(order_by = Version, n = 1, with_ties = FALSE) |>
        ungroup() |>
        select(Version, git_branch)
    
    countTbl <- countTbl |>
        mutate(version = package_version(as.character(version))) |>
        left_join(branchTbl, by = c("version" = "Version")) |>
        arrange(version, node)
    
    if (!is.null(branch)){
        if(!branch %in% countTbl$git_branch){
            message(sprintf("Branch: '%s' Not Found after filtering for '%s'.\n  Please check spelling and capitalization against list_all_branches()",
                            branch, packagename))
        }else{
            countTbl <- countTbl |> filter(git_branch %in% branch)
        }
    }

    return(countTbl)
}


#' @title Get rough timeline overview of package failing events
#'
#' @description Get rough timeline overview of a package failing events on a
#' specific builder
#'
#' @details This function attempts to give an overview of how long a package has
#' been failing on a given builder. The time to treat failures as a single event
#' is distinguished by consecutive days and by the failure_cluster_hours
#' argument. Currently there is a different cadence of builds for release
#' compared to devel so this allows the flexibility to define the timeline.
#' Let's use the following example for interpretation of the result table:
#' ```
#'   version episode       first_failure        last_failure n_failures
#'2   3.1.0       3 2025-12-15 02:57:57 2025-12-16 21:29:20          3
#'3   3.1.0       2 2025-12-13 02:32:33 2025-12-13 02:32:33          1
#'4   3.1.0       1 2025-12-08 21:30:14 2025-12-11 21:27:29          4
#'              stages statuses
#'2 buildsrc, checksrc    ERROR
#'3           checksrc    ERROR
#'4           buildsrc    ERROR
#' ```
#' The package version being built on the build system for this package is
#' 3.1.0. The episode refers to the number of failing episodic events. If we
#' look at the episode 1, the package started failing on December 8 and
#' continued to fail until December 11. During that time period there were 4
#' number of failures and they occurred during the build stage. The time between
#' the last failure of the first episode and the second episode is from December
#' 11 to December 13; we can infer then that the package built cleanly on
#' December 12. During episode 2 the package failed solely on December 13 but
#' during the check stage. Episode 3 began on Dec 15 and lasted until December
#' 16 and had 3 failures but they occurred in either the build or check stage.
#'
#' This hopefully can help determine if package failures are intermittent and
#' how long a package is failing on a given builder.
#' 
#' @param packagename a single valid Bioconductor package name
#' @param builder name of a valid Bioconductor build machine (ie 'nebbiolo1')
#' @param failure_cluster_hours numeric indicating the number of hours to treat
#' failures as a single event
#'
#' @return tibble of failure event information or NULL if package or builder is not found
#'
#' @aliases package_failure_over_time
#'
#' @seealso get_bbs_table
#'
#' @author Lori Shepherd
#'
#' @examples
#' package_failures_over_time("BiocFileCache", "nebbiolo1", 24)
#'
#' @export
#' 
package_failures_over_time <- function(packagename, builder, failure_cluster_hours = 72) {

    stopifnot(is.character(packagename), length(packagename) == 1L)
    stopifnot(is.character(builder), length(builder) == 1L)
    stopifnot(is.numeric(failure_cluster_hours), length(failure_cluster_hours) == 1L)

    summaryTbl <- get_bbs_table("build_summary")

    if (!(packagename %in% summaryTbl$package)) {
        message(sprintf("Package: '%s' Not Found.\n  Please check spelling and capitalization",
                        packagename))
        return(NULL)
    }

    if (!(builder %in% summaryTbl$node)) {
        message(sprintf("Builder: '%s' Not Found.\n  Please check spelling and capitalization against list_all_builders()",
                        builder))
        return(NULL)
    }

    pkgTbl <- summaryTbl |>
        filter(package == packagename,
               status %in% c("ERROR", "TIMEOUT"),
               node == builder) |>
        mutate(version = as.package_version(version)) |>
        arrange(startedat)   # oldest first for correct gap calculation

    if (nrow(pkgTbl) == 0) {
        message("No failing builds found for this package and builder.")
        return(NULL)
    }

    pkgEpisodes <- pkgTbl |>
        group_by(version) |>
        mutate(
            gap_hours = as.numeric(difftime(startedat, lag(startedat), units = "hours")),
            gap_days  = as.numeric(difftime(as.Date(startedat), lag(as.Date(startedat)), units = "days")),
            episode = cumsum(is.na(gap_hours) | (gap_hours > failure_cluster_hours & gap_days > 1))
        ) |>
        ungroup()

    episodeSummary <- pkgEpisodes |>
        group_by(version, episode) |>
        summarise(
            first_failure = min(startedat),
            last_failure  = max(startedat),
            n_failures    = n(),
            stages        = paste(sort(unique(stage)), collapse = ", "),
            statuses      = paste(sort(unique(status)), collapse = ", "),
            .groups = "drop"
        ) |>
        arrange(desc(first_failure))

    
    return(episodeSummary)
}


#' @title Get currently active Bioconductor git branches
#'
#' @description Get currently active Bioconductor git branches
#'
#' @details Get currently active Bioconductor git branches
#'
#' @param infoTbl downloaded paraquet info table retrieved from
#' get_bbs_table('info') that includes git_branch information.
#'
#' @return character(2) 'devel' and RELEASE_X_Y value eqivalent to current
#' release.
#'
#' @seealso get_bbs_table
#'
#' @aliases get_latest_branches
#'
#' @examples
#' get_latest_branches()
#' 
#' @export
get_latest_branches <- function(infoTbl=NULL) {

    if(is.null(infoTbl)) infoTbl <- get_bbs_table("info")

    stopifnot("git_branch" %in% names(infoTbl), inherits(infoTbl, "data.frame"))
    
    release_branches <- grep("^RELEASE", infoTbl$git_branch, value = TRUE)
    if (length(release_branches) > 0) {
        release_versions <- as.numeric(gsub("RELEASE_(\\d+)_(\\d+)", "\\1\\2", release_branches))
        latest_release <- release_branches[which.max(release_versions)]
    } else {
        latest_release <- character(0)
    }
    
    c("devel", latest_release)
}


#' @title Get list of all Bioconductor git branches
#'
#' @description Get list of allBioconductor git branches
#'
#' @details Get list of all Bioconductor git branches
#'
#' @param infoTbl downloaded paraquet info table retrieved from
#' get_bbs_table('info') that includes git_branch information.
#'
#' @return character() vector listing all options for "branch"
#'
#' @seealso get_bbs_table
#'
#' @aliases list_all_branches
#'
#' @examples
#' list_all_branches()
#' 
#' @export
list_all_branches <- function(infoTbl=NULL) {

    if(is.null(infoTbl)) infoTbl <- get_bbs_table("info")

    stopifnot("git_branch" %in% names(infoTbl), inherits(infoTbl, "data.frame"))
    
    infoTbl |> distinct(git_branch) |> pull(git_branch)
}


#' @title Get list of all Bioconductor builders
#'
#' @description Get list of allBioconductor builders
#'
#' @details Get list of all Bioconductor builders
#'
#' @param summaryTbl downloaded paraquet build_summary table retrieved from
#' get_bbs_table('build_summary') that includes builder (node)  information.
#'
#' @return character() vector listing all options for "builder"
#'
#' @seealso get_bbs_table
#'
#' @aliases list_all_builders
#'
#' @examples
#' list_all_builders()
#' 
#' @export
list_all_builders <- function(summaryTbl=NULL) {

    if(is.null(summaryTbl)) summaryTbl <- get_bbs_table("build_summary")

    stopifnot("node" %in% names(summaryTbl), inherits(summaryTbl, "data.frame"))
    
    summaryTbl |> distinct(node) |> pull(node)
}


#' @title Get a report for any given day from the Bioconductor Build System
#'
#' @description Get a report for any given day from the Bioconductor Build
#' System, optionally specifying a git branch or build machine name.
#'
#' @details Utilizes data from the build_summary and info parquet files to
#' retrieve a reconstructed Bioconductor Build Report for any given
#' day. Optionally a given build machine name or Bioconductor branch may be
#' specified for further filtering. By default it will use today's date and
#' retrieve both devel and current release for all Bioconductor build machines.
#' NOTE: Traditionally Bioconductor splits builds into software,
#' data-experiment, annotation, workflow, books; this distinction is not
#' available in these functions and packages regardless of 'type' will be
#' included.
#' The returned tibble includes: package name, build node, build stage, package
#' version number, status of stage, when the run started and ended, the command
#' the builder ran, branch of bioconductor and relevant git commit information. 
#' 
#' @param build_date Date of report to retrieve (eg. '2025-12-19')
#' @param branch a bioconductor branch (eg. 'devel', 'RELEASE_3_22')
#' @param builder name of a valid Bioconductor build machine (ie 'nebbiolo1')
#'
#' @return tibble representing a Bioconductor build report for a given day
#'
#' @aliases get_build_report
#'
#' @seealso get_bbs_table
#' 
#' @author Lori Shepherd
#'
#' @examples
#' get_build_report("2025-12-29", branch="RELEASE_3_22", builder="nebbiolo2")
#'
#' @export

get_build_report <- function(build_date = Sys.Date(), branch = NULL, builder = NULL) {

    stopifnot(inherits(build_date, c("Date", "character")))
  
    summaryTbl <- get_bbs_table("build_summary")
    infoTbl <- get_bbs_table("info")
    
    build_date <- as.Date(build_date)
    
    dailyTbl <- summaryTbl |> 
        filter(as.Date(startedat) == build_date)
    
    if (nrow(dailyTbl) == 0) {
        message("No builds found on this date.")
        return(NULL)
    }
  
    if (is.null(branch)) {
        branch_filter <- get_latest_branches(infoTbl)
    } else {
        branch_filter <- branch
    }
  
    info_filtered <- infoTbl |> 
        filter(git_branch %in% branch_filter) |> 
        group_by(Package, git_branch) |> 
        slice_max(package_version(Version), n = 1, with_ties = FALSE) |> 
        ungroup() |> 
        select(Package, Version, git_branch, git_last_commit, git_last_commit_date)
  
    daily_report <- dailyTbl |> 
        inner_join(info_filtered, by = c("package" = "Package", "version" = "Version"))
  
    if (!is.null(builder)) {
        daily_report <- daily_report |> filter(node == builder)
        if (nrow(daily_report) == 0) {
            message(sprintf("No builds found for builder '%s' on %s.", builder, build_date))
            return(NULL)
        }
    }
    stage_levels <- c("install", "buildsrc", "checksrc")
    
    daily_report <- daily_report |> 
        mutate(stage = factor(stage, levels = stage_levels),
               version = package_version(version)) |> 
        arrange(git_branch, package, version, node, stage)
    
    daily_report
}


#' @title Get list of failing packages for the most recent build
#'
#' @description Get a list of failing (ERROR/TIMEOUT) packages for any given
#' branch and builder
#'
#' @details Utilizes data from the build_summary and info parquet files to list
#' all packages failing (ERROR/TIMEOUT) in any stage of the process. Optionally
#' a Bioconductor git branch can be specified and by default will do both the
#' current devel and release. The report can be further filtered by optionally
#' specifying a build machine name.
#' The returned tibble includes: git branch, package name, package version
#' number, build machine node, collapsed column of stages and statuses.
#'
#' @param branch a bioconductor branch (eg. 'devel', 'RELEASE_3_22')
#' @param builder name of a valid Bioconductor build machine (ie 'nebbiolo1')
#'
#' @return a tibble of failing packages
#'
#' @aliases get_failing_packages
#'
#' @seealso get_bbs_table
#'
#' @author Lori Shepherd
#'
#' @examples
#' get_failing_packages("RELEASE_3_22")
#' get_failing_packages("RELEASE_3_22", "nebbiolo2")
#'
#' @export
get_failing_packages <- function(branch = NULL, builder = NULL) {

    summaryTbl <- get_bbs_table("build_summary")
    infoTbl <- get_bbs_table("info")
  
    if (is.null(branch)) {
        branch_filter <- get_latest_branches(infoTbl)
    } else {
        branch_filter <- branch
    }
  
    info_filtered <- infoTbl |> 
        filter(git_branch %in% branch_filter) |> 
        group_by(Package, git_branch) |> 
        slice_max(package_version(Version), n = 1, with_ties = FALSE) |> 
        ungroup() |> 
        select(Package, Version, git_branch)
  
    failures <- summaryTbl |> 
        filter(status %in% c("ERROR", "TIMEOUT")) |> 
        inner_join(info_filtered,
                   by = c("package" = "Package", "version" = "Version"),
                   relationship = "many-to-many")
  
    if (!is.null(builder)) {
        failures <- failures |> filter(node %in% builder)
    }
  
    if (nrow(failures) == 0) {
        message("No failing packages found for the specified branch(es) and node(s).")
        return(NULL)
    }
    

    failures |> 
        mutate(version = package_version(version)) |>
        group_by(git_branch, package, version, node) |> 
        summarise(
            stages   = paste(sort(unique(stage)), collapse = ", "),
            statuses = paste(sort(unique(status)), collapse = ", "),
            .groups  = "drop"
        ) |> 
        arrange(git_branch, package, version, node)


}

test_that("get_package_release_info", {

    expect_true(is.null(suppressMessages(get_package_release_info("NotAValidPackage"))))
    expect_error(get_package_release_info())
    expect_error(get_package_release_info(c("BiocFileCache", "AnnotationHub")))
    releaseInfo <- get_package_release_info("BiocFileCache")
    expect_true(all(c("tbl", "data.frame") %in% class(releaseInfo)))
    expect_true(all(c("Package", "Version", "git_branch", "git_last_commit",
    "git_last_commit_date") %in% names(releaseInfo)))
    
})

test_that("get_package_build_results", {

    expect_true(is.null(suppressMessages(get_package_build_results("NotAValidPackage")))) 
    expect_true(is.null(suppressMessages(get_package_build_results("BiocFileCache", branch="NotAValidBranch"))))
    expect_error(get_package_build_results(c("BiocFileCache", "AnnotationHub")))
    expect_error(get_package_build_results())
    expect_error(get_package_build_results("BiocFileCache", branch=c("devel", "RELEASE_3_22")))
    devBuild <- get_package_build_results("BiocFileCache")
    expect_true((devBuild |> distinct(git_branch) |> pull()) == "devel")
    expect_true((devBuild |> distinct(package) |> pull()) == "BiocFileCache")
    expect_true(all(c("package", "node", "stage", "version", "status", "git_branch", "git_last_commit",
                      "git_last_commit_date") %in% names(devBuild)))
    releaseBuild <-  get_package_build_results("BiocFileCache", "RELEASE_3_22")
    expect_true((releaseBuild |> distinct(git_branch) |> pull()) == "RELEASE_3_22")
})

test_that("package_error_count", {

    expect_true(is.null(suppressMessages(package_error_count("NotAValidPackage")))) 
    expect_error(package_error_count())
    allError <- package_error_count("BiocFileCache")
    expect_true((allError |> distinct(git_branch) |> nrow()) > 1)
    expect_true(all(c("node", "version", "stage", "count_total", "count_error", "git_branch") %in% names(allError)))
    expect_true(all(allError$count_total >= allError$count_error))
    branchError <- package_error_count("BiocFileCache", branch="RELEASE_3_22")
    expect_true((branchError |> distinct(git_branch) |> pull()) == "RELEASE_3_22")
    OSError <- package_error_count("BiocFileCache", builder="nebbiolo2", branch="RELEASE_3_22")
    expect_true((OSError |> distinct(git_branch) |> pull()) == "RELEASE_3_22")
    expect_true((OSError |> distinct(node) |> pull()) == "nebbiolo2")
    expect_true(all(OSError$count_total >= OSError$count_error))

    tempTbl <- suppressMessages(package_error_count("BiocFileCache", builder="bogus"))
    expect_true((tempTbl |> distinct(node) |> nrow()) > 1)
    expect_false("bogus" %in% tempTbl$node)
    expect_true((tempTbl |> distinct(git_branch) |> nrow()) > 1)

    tempTbl <- suppressMessages(package_error_count("BiocFileCache", branch="bogus"))
    expect_true((tempTbl |> distinct(git_branch) |> nrow()) > 1)
    expect_false("bogus" %in% tempTbl$git_branch)
    
    
})


test_that("package_failures_over_time", {

    expect_error(package_failures_over_time())
    expect_error(package_failures_over_time("NotAValidPackage"))
    expect_error(package_failures_over_time("NotAValidPackage", "NotAValidBuilder", "nottime"))
    expect_true(is.null(suppressMessages(package_failures_over_time("NotAValidPackage", "NotAValidBuilder")))) 
    expect_true(is.null(suppressMessages(package_failures_over_time("BiocFileCache", "NotAValidBuilder")))) 
    failTbl <- package_failures_over_time("BiocFileCache", "nebbiolo1", 24)
    expect_true(all(c("version", "episode", "first_failure", "last_failure","n_failures", "stages", "statuses") %in% names(failTbl)))
    expect_true(all(failTbl$first_failure <= failTbl$last_failure))

})

test_that("get_latest_branches", {

    expect_error(get_latest_branches(data.frame()))
    res <- get_latest_branches()
    expect_true(res[1] == "devel")
    expect_true(length(res)==2L)
    
})


test_that("list_all_branches", {

    expect_error(get_all_branches(data.frame()))
    res <- list_all_branches()
    expect_true(length(res) > 0)
    expect_true(is.character(res))
    expect_true(all(get_latest_branches() %in% res))
})


test_that("list_all_builders", {

    expect_error(get_all_builders(data.frame()))
    res <- list_all_builders()
    expect_true(length(res) > 0)
    expect_true(is.character(res))
})


test_that("get_build_report", {
    expect_error(get_build_report("NotADate"))
    res <-  get_build_report("2025-12-29", branch="RELEASE_3_22")
    expect_true(all(c("package", "node", "version", "stage", "status", "git_branch") %in% names(res)))
    release_vl <- res |> select(git_branch) |> distinct() |> pull()
    expect_true(length(release_vl) == 1L)
    expect_true("RELEASE_3_22" %in% release_vl)
    date_start <- res |> select(startedat) |> pull()
    expect_true(all(startsWith(as.character(date_start),"2025-12-29" )))
    build_nodes <- res |> select(node) |> distinct() |> pull()
    expect_true(length(build_nodes) >= 1L)
    res <-  get_build_report("2025-12-29", branch="RELEASE_3_22", builder="nebbiolo2")
    release_vl <- res |> select(git_branch) |> distinct() |> pull()
    expect_true(length(release_vl) == 1L)
    expect_true("RELEASE_3_22" == release_vl)
    build_nodes <- res |> select(node) |> distinct() |> pull()
    expect_true(length(build_nodes) == 1L)
    expect_true("nebbiolo2" == build_nodes) 

})

test_that("get_failing_packages", {
    res <- get_failing_packages()
    release_vl <- res |> select(git_branch) |> distinct() |> pull()
    expect_true(length(release_vl) == 2L)
    expect_true("devel" %in% release_vl)
    build_nodes <- res |> select(node) |> distinct() |> pull()
    expect_true(length(build_nodes) >= 1L)
    res <- get_failing_packages("RELEASE_3_22")
    release_vl <- res |> select(git_branch) |> distinct() |> pull()
    expect_true(length(release_vl) == 1L)
    expect_true("RELEASE_3_22" == release_vl)
    build_nodes <- res |> select(node) |> distinct() |> pull()
    expect_true(length(build_nodes) >= 1L)
    res <- get_failing_packages("RELEASE_3_22", "nebbiolo2")
    release_vl <- res |> select(git_branch) |> distinct() |> pull()
    expect_true(length(release_vl) == 1L)
    expect_true("RELEASE_3_22" == release_vl)
    build_nodes <- res |> select(node) |> distinct() |> pull()
    expect_true(length(build_nodes) == 1L)
    expect_true("nebbiolo2" == build_nodes) 
    
})

## File for Internal Helpers

remote_url_base <- "https://mghp.osn.xsede.org/bir190004-bucket01/BiocBuildReports/"

.getCacheLocation <- function(){
    tools::R_user_dir(package = "BiocBuildReporter", which = "cache")
}

.makeLocalCache <- function(){
    cache <- .getCacheLocation()
    bfc <- BiocFileCache::BiocFileCache(cache)
}

.checkInCache <- function(bfc, tblname){
    res <- bfcquery(bfc, query=tblname, field="rname")
    as.logical(nrow(res))
}

.addToCache <- function(bfc, tblname){
    fpath <- paste0(remote_url_base, tblname, ".parquet")
    bfcadd(bfc, rname=tblname, fpath=fpath) 
}

.getCacheId  <- function(bfc, tblname){
    res <- bfcquery(bfc, query=tblname, field="rname")
    bfcrid(res)
}

.getUrl <- function(tblname, useLocal, updateLocal){

    if(useLocal){
        bfc <- .makeLocalCache()
        fnd <- .checkInCache(bfc, tblname)
        if(fnd){
            message(sprintf("Reading local file for '%s'", tblname))
            rid <- .getCacheId(bfc, tblname)
            url <- ifelse(updateLocal,
                          bfcdownload(bfc, rid=rid, ask=FALSE),
                          bfcpath(bfc, rids=rid))  
        }else{
            message(sprintf("Local file for '%s' Not Found. Downloading...", tblname))
            url <- .addToCache(bfc, tblname)
        }
        
    }else{
        
        message(sprintf("reading '%s' parquet file...\n", tblname),
                "    Initial read may take a few minutes\n",
                "    Subsequent access through cache will be instantaneous.")
        
        s3fs <- S3FileSystem$create(
                                 anonymous = TRUE,
                                 endpoint_override = "https://mghp.osn.xsede.org"
                             )
        file_path <- paste0("bir190004-bucket01/BiocBuildReports/", tblname, ".parquet")
        url <- s3fs$OpenInputFile(file_path)

    }
    url
}

utils::globalVariables(
    c("Package", "Version", "endedat", "episode", "first_failure",
      "gap_days", "gap_hours", "git_branch", "git_last_commit",
      "git_last_commit_date", "last_modified", "node", "old_last_modified",
      "package", "propagate", "stage", "startedat", "status", "version_obj")
    )

#' @title Custom Environment to store downloaded parquet tibbles.
#'
#' @rdname bbs_cache
#' @name bbs_cache
#' 
#' @description A custom environment to store remotely read parquet files of
#' Bioconductor build reports. Once one of the three valid parquet files are
#' read in, the table is cached in this environment for quick easy reference
#' either to use ad hocly by the user or to be used within pre-defined queries
#' provided by this package.
#'
#' @aliases .bbs_cache
#'
#' @format An environment
.bbs_cache <- new.env(parent = emptyenv())


#' @import arrow
#' @import BiocFileCache
#'
#' @title Retrieve a remotely read parquet file of Bioconductor build data
#'
#' @description Bioconductor Build results are at times cumbersome to work with and
#' interrogate. Parquet files of condensed reports are provided remotely. These
#' functions read the remote parquet files using the arrow package and return a
#' tibble. Once retrieved the table is stored in a custom environment for easy
#' quick reference either to use ad hocly or within pre-defined queries provided
#' by this package. If useLocal is chosen, the remote parquet files are
#' downloaded locally and cached using BiocFileCache. If already downloaded, the
#' local file is utilized unless updateLocal is also TRUE in which case the file
#' will be redownloaded to the local location before reading into the
#' session. If useLocal is FALSE, the files will be read remotely.
#'
#' @details The get_bbs_table returns a tibble but is also saved in a custom
#' environment for quick easy subsequent reference. There are three tables
#' available and can be specified in get_bbs_table: build_summary, info, or
#' propagation_status. build_summary returns a tibble with the results of each
#' stage of the Bioconductor build process. info returns a tibble with
#' information regarding the package maintainer and git commit
#' status. propagation_status returns a tibble with data on if a package
#' propagated to the community.
#'
#' @param tblname A valid parquet file name. Currently available: build_summary,
#' info, or propagation_status.
#'
#' @param useLocal A logical indicating if files should be read/downloaded
#' locally or read remotely. 
#'
#' @param updateLocal A logical indicating if using locally cached files, if the
#' files should be re-downloaded/updated. Theorically build reports are updated
#' daily so the local data may be out of date if not updated.
#'
#' @return returns a tibble. 
#'
#' @author Lori Shepherd
#'
#' @aliases get_bbs_table
#'
#' @seealso get_all_bbs_tables
#' 
#' @examples
#' info <- get_bbs_table("info")
#' info
#'
#' library(dplyr)
#' # all the package Lori Shepherd maintains
#' info |> filter(Maintainer == "Lori Shepherd") |> distinct(Package)
#'
#'
#' @export
get_bbs_table <- function(tblname=c("build_summary", "info",
                                    "propagation_status"),
                          useLocal=TRUE,
                          updateLocal=FALSE){
    
    tblname <- match.arg(tblname)
    stopifnot(is.logical(useLocal), length(useLocal)==1L,
              is.logical(updateLocal), length(updateLocal)==1L)
    
    if (exists(tblname, envir = .bbs_cache) & !updateLocal & useLocal) {
        message(sprintf("Using cached table '%s'", tblname))
        return(get(tblname, envir = .bbs_cache))
    }  
    
    url <- .getUrl(tblname, useLocal, updateLocal)
    
    tbl <-
        tryCatch(
            arrow::read_parquet(url),
            error = function(e){
                warning(
                    sprintf("Could not read '%s' from location (%s)",
                            tblname, conditionMessage(e)),
                    call. = FALSE
                )
                    return(NULL)
            })
    
    if (!is.null(tbl)) {
        assign(tblname, tbl, envir = .bbs_cache)
    }
  
    tbl
}

#' @title Retrieve all available Bioconductor build system parquet files
#'
#' @description Retrieve all available parquet tables at once.
#'
#' @param assign_to_global logical(1) indicating if downloaded tables should be
#' assigned to the current global environment for access. Default is FALSE and
#' only assigned to custom environment through call to get_bbs_table.
#'
#' @param useLocal A logical indicating if files should be read/downloaded from
#' a local file or read remotely. 
#'
#' @param updateLocal A logical indicating if using locally cached files, if the
#' files should be re-downloaded/updated. Theorically build reports are updated
#' daily so the local data may be out of date if not updated.
#'
#' @return invisible. tables are cached to custom environment unless
#' assign_to_global is TRUE.
#'
#' @aliases get_all_bbs_tables
#'
#' @details This function is a quick wrapper around get_bbs_table to download
#' all available parquet files as tibbles.  By default no value is returned and
#' the tibbles are stored to the custom environment and can be easily accessed
#' through a call to get_bbs_table. If useLocal and updateLocal are TRUE, this
#' also becomes an easy helper to update/redownloaded all files to the locally
#' downloaded location.
#' 
#' @author Lori Shepherd
#'
#' @seealso get_bbs_table
#' 
#' @examples
#' get_all_bbs_tables()
#'
#' # Now that all are downloaded. Retrieve cached build_summary tibble
#' build_summary <- get_bbs_table("build_summary")
#' 
#' library(stringr)
#' library(dplyr)
#' # find the times the package BiocFileCache failed on linux nebbiolo builders
#' \donttest{
#' build_summary |> 
#'   filter(
#'     package == "BiocFileCache", 
#'     status %in% c("TIMEOUT", "ERROR"), 
#'     str_starts(node, "nebbiolo")
#'   ) 
#' }
#' @export 
get_all_bbs_tables <- function(assign_to_global = FALSE,
                               useLocal=TRUE,
                               updateLocal=FALSE) {

    stopifnot(is.logical(assign_to_global), length(assign_to_global)==1L,
              is.logical(useLocal), length(useLocal)==1L,
              is.logical(updateLocal), length(updateLocal)==1L)
    
    table_names <- c("build_summary", "info", "propagation_status")
    tables <- list()
  
    for (tblname in table_names) {
        tbl <- get_bbs_table(tblname, useLocal=useLocal, updateLocal=updateLocal)
        tables[[tblname]] <- tbl
        
        if (assign_to_global && !is.null(tbl)) {
            assign(tblname, tbl, envir = .GlobalEnv)
            message(sprintf("Table '%s' assigned to global environment", tblname))
        }
    }
    
    invisible(tables)    
}

test_that("get_bbs_table_works", {

    if(exists("info", envir = .bbs_cache)) rm("info", envir = .bbs_cache)
    
    expect_false(exists("info", envir = .bbs_cache))
    info <- get_bbs_table("info")
    expect_true(exists("info", envir = .bbs_cache))
    expect_true(all(c("tbl", "data.frame") %in% class(info)))
    expect_true(all(c("Package", "Version", "git_branch", "git_last_commit", "git_last_commit_date") %in% names(info)))
    expect_error(get_bbs_table("NotValidTable"))
    rm("info", envir = .bbs_cache)
})

test_that("get_all_bbs_tables_work", {

    if(exists("info", envir = .bbs_cache)) rm("info", envir = .bbs_cache)
    if(exists("propagation_status", envir = .bbs_cache)) rm("propagation_status", envir = .bbs_cache)
    if(exists("build_summary", envir = .bbs_cache)) rm("build_summary", envir = .bbs_cache)
    if(exists("info")) rm("info")
    if(exists("build_summary")) rm("build_summary")
    if(exists("propagation_status")) rm("propagation_status")
   
    expect_false(exists("info", envir = .bbs_cache))
    expect_false(exists("propagation_status", envir = .bbs_cache))
    expect_false(exists("build_summary", envir = .bbs_cache))
    expect_false(exists("propagation_status"))
    expect_false(exists("info"))
    expect_false(exists("build_summary"))   
    tmpTbls <- get_all_bbs_tables()
    expect_true(all(c("build_summary", "info", "propagation_status") %in% names(tmpTbls)))
    expect_true(exists("info", envir = .bbs_cache))
    expect_true(exists("propagation_status", envir = .bbs_cache))
    expect_true(exists("build_summary", envir = .bbs_cache))
    expect_error(get_all_bbs_tables("NotValid"))
    get_all_bbs_tables(assign_to_global=TRUE)
    expect_true(exists("propagation_status"))
    expect_true(exists("info"))
    expect_true(exists("build_summary"))
    expect_true(all(c("package", "node", "stage", "version", "status", "startedat", "endedat") %in% names(build_summary)))
   
   
})

test_that("remote_connections_available",{

    remote_url_base <- "https://mghp.osn.xsede.org/bir190004-bucket01/BiocBuildReports/"
    
    url_exists <- function(url) {
        con <- url(url)
        suppressWarnings({
            tryCatch({
                open(con)
                close(con)
                TRUE
            }, error = function(e) {
                try(close(con), silent = TRUE)
                FALSE
            })
        })
    }
    expect_error(url_exists("notthere"))
    expect_true(url_exists("https://bioconductor.org"))
    
    tbls <- c("build_summary", "info", "propagation_status")
    for(tbl in tbls){
        url <- paste0(remote_url_base, tbl, ".parquet")
        expect_true(url_exists(url))
    }
    
})

---
title: "BiocBuildReporter Data Use Cases"
author: "BiocBuildReporter Team"
format: html
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{BiocBuildReporter Data Use Cases}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 6,
  warning = FALSE,
  message = FALSE
)
```

# Background

The BiocBuildReporter package provides access to years of Bioconductor build
system data, representing a comprehensive record of package builds across:

- **Thousands of packages** in the Bioconductor ecosystem
- **Multiple R versions** spanning several years of development
- **Multiple platforms** including Linux, macOS, and Windows
- **Different build stages** (install, build, check)

The Bioconductor build system runs regularly, testing all packages to ensure
they meet quality standards and work correctly across different platforms. This
dataset captures the results of these builds, including:

- Build status (OK, WARNING, ERROR, TIMEOUT)
- Package version information
- Git commit information
- Maintainer details
- Propagation status to the community

This vignette demonstrates how to use BiocBuildReporter functions to explore and
analyze this rich dataset to understand:

- Package build history and stability
- Platform-specific issues
- Package growth over time
- Common failure patterns

The Bioconductor Build Report Logs are being processed by
[BiocBuildDB](https://github.com/seandavi/BiocBuildDB). The BiocBuildDB creates
parquet files read by this package for analysis.

# Installation and Loading

`BiocBuildReporter` is a _Bioconductor_ package and can be installed through
`BiocManager::install()`.

```{r, install, eval = FALSE}
if (!"BiocManager" %in% rownames(installed.packages()))
     install.packages("BiocManager")
BiocManager::install("BiocBuildReporter", dependencies=TRUE)
```

After the package is installed, it can be loaded into _R_ workspace by

```{r, library, results='hide', warning=FALSE, message=FALSE}
library(BiocBuildReporter)
```

We will also load libraries need to run this vignette:

```{r, setup}
library(BiocBuildReporter)
library(dplyr)
library(ggplot2)
library(tidyr)
```

# Accessing Bioconductor Build Report Data

## Getting All Available Tables

The simplest way to start is to download all available data tables. The
`get_all_bbs_tables()` function retrieves all three parquet files containing
Bioconductor build data:

```{r, get_all_tables}
# Download all available tables
# This will cache the tables for quick subsequent access
get_all_bbs_tables()
```

The function downloads three tables:

1. **build_summary**: Results of each build stage for every package
2. **info**: Package metadata including version, maintainer, and git information
3. **propagation_status**: Information about package propagation to the community

## Getting Individual Tables

You can also retrieve individual tables using `get_bbs_table()`:

```{r, get_individual_table}
# Get the build summary table
build_summary <- get_bbs_table("build_summary")

# Get the info table
info <- get_bbs_table("info")

# Get the propagation status table
propagation_status <- get_bbs_table("propagation_status")
```

Once downloaded, subsequent calls to these functions will use cached data,
making analysis much faster.

## Remote Read vs Local Download

The package allows the option to either download locally and read from a locally
saved BiocFileCache, or to read files remotely. The default is to download
locally and use a cached version of the files for analysis. To read remotely,
calls to `get_bbs_table` or `get_all_bbs_tables` should be altered to use the
argument `useLocal=FALSE`.

```{r, read_remote}
info <- get_bbs_table("info", useLocal=FALSE)
```

Bioconductor devel reports are daily, meaning cached data can get out of date
quickly. If using locally cached data, you can update/re-download the files
using the argument `updateLocal=TRUE`. The default is not to update.

```{r, updateTables}
info <- get_bbs_table("info", useLocal=TRUE, updateLocal=TRUE)
```


# Package-Specific Queries

This section shows usage of BiocBuildReporter provided helper functions for
package specific analysis.

## Package Release Information

The `get_package_release_info()` function retrieves version and git information
for a package across all Bioconductor releases:

```{r, package_release_info}
# Get release information for BiocFileCache
bfc_releases <- get_package_release_info("BiocFileCache")
bfc_releases
```

This shows:

- Package versions across different Bioconductor releases
- Git branches (devel, RELEASE_3_22, etc.)
- Git commit hashes
- Last commit dates

This is useful for tracking when a package was updated in different Bioconductor
releases. It also is useful for mapping which git commit correpsonds to the
currently available version in a given Bioconductor release.

## Package Build Results

The `get_package_build_results()` function retrieves latest build status
information for a package for all available builders on a given Bioconductor
branch:

```{r, get_package_build_results}
# Get build results for BiocFileCache on branch RELEASE_3_22
get_package_build_results("BiocFileCache", branch="RELEASE_3_22")
```

This shows:

- Node (builder machine name)
- Build stage (install, build, check)
- Package version
- Build status (OK, WARNING, ERROR, TIMEOUT)
- Date of completion
- Git branch
- Git commit hashes
- Last commit dates

This is useful for showing the last known status of a package on a given
release for all active builders.

## Package Error Counts

The `package_error_count()` function provides statistics on how often a package
has failed during builds:

```{r, package_error_count}
# Get error counts for BiocFileCache
bfc_errors <- package_error_count("BiocFileCache")
bfc_errors

# Filter to a specific branch
bfc_errors_release <- package_error_count("BiocFileCache", branch = "RELEASE_3_22")
bfc_errors_release

# Filter to a specific builder
bfc_errors_builder <- package_error_count("BiocFileCache", 
                                          builder = "nebbiolo2", 
                                          branch = "RELEASE_3_22")
bfc_errors_builder
```

This returns:

- Node (builder machine name)
- Package version
- Build stage (install, build, check)
- Total number of runs
- Total number of errors
- Git branch

For the devel branch, you can filter to the most recent version:

```{r, filter_devel_errors}
# Get devel errors
dev_errors <- package_error_count("BiocFileCache", branch = "devel")

# Filter to current devel version
dev_errors |> filter(version == max(version))
```

This is useful for comparing how often a package failed during a given stage vs
how often it attempted that stage. It gives an overview of frquency of failures.

## Package Failures Over Time

The `package_failure_over_time()` function gives and overview of how long a
package has been failing on a given builder:

```{r, package_failure_over_time}
# Get failure events for BiocFileCache on nebbiolo1 and
# group events in a 24 hour period
package_failures_over_time("BiocFileCache", "nebbiolo1", 24)
```

This shows:

- Package version
- Sequential count of number of failure episodes
- Time of first event failure
- Time of last event failure
- Number of failures during that episode
- Stages of failures
- Status of stages

This is useful to track the length of failure events to determine if it is
intermittent or consistent. The grouping of events is given as an argument, in
this example we used 24 hours. This is to account for branches having different
build cadenances and allowing sequential builds to be potentially grouped
together.

# Exploratory Data Analysis

This section provides examples of utilizing the report tables for broader
analysis and queries.

## Package Growth Over Time

Let's explore how the number of Bioconductor packages has grown over time:

```{r, package_growth}
# Get info table
info <- get_bbs_table("info")

# Count unique packages by branch
package_counts <- info |>
  group_by(git_branch) |>
  summarise(
    n_packages = n_distinct(Package),
    .groups = "drop"
  ) |>
  arrange(desc(n_packages))

# Display the counts
package_counts

# Visualize package counts by branch
ggplot(package_counts, aes(x = reorder(git_branch, n_packages), y = n_packages)) +
  geom_col(fill = "steelblue") +
  coord_flip() +
  labs(
    title = "Number of Packages by Bioconductor Branch",
    x = "Branch",
    y = "Number of Packages"
  ) +
  theme_minimal()
```

## Build Status Distribution

Understanding the distribution of build statuses helps identify overall system health:

```{r, build_status}
# Get build summary table
build_summary <- get_bbs_table("build_summary")

# Count build statuses
status_counts <- build_summary |>
  count(status) |>
  arrange(desc(n))

status_counts

# Visualize status distribution
ggplot(status_counts, aes(x = reorder(status, n), y = n)) +
  geom_col(aes(fill = status)) +
  scale_fill_manual(values = c(
    "OK" = "green3",
    "WARNING" = "orange",
    "ERROR" = "red",
    "TIMEOUT" = "darkred"
  )) +
  coord_flip() +
  labs(
    title = "Distribution of Build Statuses",
    x = "Status",
    y = "Count"
  ) +
  theme_minimal() +
  theme(legend.position = "none")
```

## Platform-Specific Analysis

Different platforms may have different build characteristics:

```{r, platform_analysis}
# Analyze build status by platform (node)
platform_status <- build_summary |>
  group_by(node, status) |>
  summarise(count = n(), .groups = "drop") |>
  group_by(node) |>
  mutate(
    total = sum(count),
    percentage = count / total * 100
  ) |>
  ungroup()

# Show error rates by platform
error_rates <- platform_status |>
  filter(status %in% c("ERROR", "TIMEOUT")) |>
  group_by(node) |>
  summarise(
    error_count = sum(count),
    total = first(total),
    error_rate = sum(percentage),
    .groups = "drop"
  ) |>
  arrange(desc(error_rate))

head(error_rates, 10)
```

## Build Stage Analysis

Understanding which build stage most often fails:

```{r, stage_analysis}
# Analyze failures by stage
stage_failures <- build_summary |>
  filter(status %in% c("ERROR", "TIMEOUT")) |>
  count(stage, status) |>
  arrange(desc(n))

stage_failures

# Visualize
ggplot(stage_failures, aes(x = stage, y = n, fill = status)) +
  geom_col() +
  scale_fill_manual(values = c("ERROR" = "red", "TIMEOUT" = "darkred")) +
  labs(
    title = "Build Failures by Stage",
    x = "Build Stage",
    y = "Number of Failures",
    fill = "Status"
  ) +
  theme_minimal()
```

## Most Problematic Packages

Identify packages with the highest error rates:

```{r, problematic_packages}
# Find packages with most errors
package_errors <- build_summary |>
  filter(status %in% c("ERROR", "TIMEOUT")) |>
  count(package, status) |>
  group_by(package) |>
  summarise(
    total_errors = sum(n),
    .groups = "drop"
  ) |>
  arrange(desc(total_errors))

# Top 10 packages with most errors
head(package_errors, 10)
```

## Maintainer Analysis

Analyze package maintenance patterns:

```{r, maintainer_analysis}
# Get unique packages per maintainer
maintainer_packages <- info |>
  group_by(Maintainer) |>
  summarise(
    n_packages = n_distinct(Package),
    packages = paste(unique(Package), collapse = ", "),
    .groups = "drop"
  ) |>
  arrange(desc(n_packages))

# Top maintainers by number of packages
head(maintainer_packages, 10)

# Distribution of packages per maintainer
ggplot(maintainer_packages, aes(x = n_packages)) +
  geom_histogram(binwidth = 1, fill = "steelblue", color = "white") +
  labs(
    title = "Distribution of Packages per Maintainer",
    x = "Number of Packages",
    y = "Number of Maintainers"
  ) +
  theme_minimal()
```

## Temporal Analysis

Analyze build patterns over time:

```{r, temporal_analysis}
# Analyze build patterns over time
build_summary <- build_summary |>
  mutate(
    date = as.Date(startedat),
    month = format(startedat, "%Y-%m")
  )

# Build activity by month
monthly_builds <- build_summary |>
  count(month) |>
  mutate(month_date = as.Date(paste0(month, "-01")))

ggplot(monthly_builds, aes(x = month_date, y = n)) +
  geom_line(color = "steelblue", linewidth = 1) +
  geom_point(color = "steelblue") +
  labs(
    title = "Build Activity Over Time",
    x = "Month",
    y = "Number of Builds"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

# Error rate over time
monthly_errors <- build_summary |>
  group_by(month) |>
  summarise(
    total = n(),
    errors = sum(status %in% c("ERROR", "TIMEOUT")),
    error_rate = errors / total * 100,
    .groups = "drop"
  ) |>
  mutate(month_date = as.Date(paste0(month, "-01")))

ggplot(monthly_errors, aes(x = month_date, y = error_rate)) +
  geom_line(color = "red", linewidth = 1) +
  geom_point(color = "red") +
  labs(
    title = "Build Error Rate Over Time",
    x = "Month",
    y = "Error Rate (%)"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))
```

# Bioconductor Report Overview

This sections demonstrates usage of BiocBuildReporter helper functions for
broader analysis.

## Get Bioconductor Build Report

The `get_build_report()` function will retrieve the Bioconductor Build Report
for any given day. Optionally you may specify a specific Bioconductor git branch
or build machine name to filter further.

```{r, get_build_report}
# Retrieves the build report for all packages on December 29, 2025
# Filtering also for RELEASE_3_22 branch and linux "nebbiolo1" build machine
get_build_report("2025-12-29", branch="RELEASE_3_22", builder="nebbiolo2")
```

This shows:

- Package name
- Node (builder machine name)
- Build stage (install, build, check)
- Package version
- Status of stage (OK, WARNING, ERROR, TIMEOUT)
- Time started
- Time completed
- Command utilized to initiate run
- Report md5 sum
- Git branch
- Git commit hashes
- Last commit dates

This function is meant to programatically reproduce a daily report for any given
day for any given builder and branch.

## Get List of Failing Packages

The `get_failing_packages()` function will return a list of all the currently
failing packages for a given branch and build machine name:

```{r, get_failing_packages}
# returns all failing packages for RELEASE_3_22 branch
# for build machine nebbolo2
get_failing_packages("RELEASE_3_22", "nebbiolo2")
```

This shows:

- Git branch
- Package name
- Package version
- Node (builder machine name)
- Build stages (install, build, check)
- Build statuses (OK, WARNING, ERROR, TIMEOUT)

This gives a quick list of all failing packages. If querying the currently
active Bioconductor branches (see `get_latest_branches()`), maintainers of these
packages should be contacted to fix their packages to avoid deprecation and
removal from Bioconductor.

# Conclusion

The BiocBuildReporter package provides powerful tools for analyzing Bioconductor build system data. This vignette demonstrated:

1. **Data Access**: Using `get_bbs_table()` and `get_all_bbs_tables()` to retrieve build data
2. **Package-Specific Queries**: Using `get_package_release_info()`, `get_package_build_results()`, `package_error_count()` and `package_failures_over_time()` to analyze individual packages
3. **Exploratory Analysis**: Examining package growth, build statuses, platform differences, and temporal patterns
4. **Bioconductor Report Overview**: Using `get_build_report()` and `get_failing_packages()`

This dataset can help package developers, maintainers, and the Bioconductor community to:

- Monitor package build health
- Identify platform-specific issues
- Track package evolution over time
- Understand common failure patterns
- Improve package quality and reliability

For more information about specific functions, see their documentation with `?function_name`.

```{r, sessionInfo}

sessionInfo()

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{get_build_report}
\alias{get_build_report}
\title{Get a report for any given day from the Bioconductor Build System}
\usage{
get_build_report(build_date = Sys.Date(), branch = NULL, builder = NULL)
}
\arguments{
\item{build_date}{Date of report to retrieve (eg. '2025-12-19')}

\item{branch}{a bioconductor branch (eg. 'devel', 'RELEASE_3_22')}

\item{builder}{name of a valid Bioconductor build machine (ie 'nebbiolo1')}
}
\value{
tibble representing a Bioconductor build report for a given day
}
\description{
Get a report for any given day from the Bioconductor Build
System, optionally specifying a git branch or build machine name.
}
\details{
Utilizes data from the build_summary and info parquet files to
retrieve a reconstructed Bioconductor Build Report for any given
day. Optionally a given build machine name or Bioconductor branch may be
specified for further filtering. By default it will use today's date and
retrieve both devel and current release for all Bioconductor build machines.
NOTE: Traditionally Bioconductor splits builds into software,
data-experiment, annotation, workflow, books; this distinction is not
available in these functions and packages regardless of 'type' will be
included.
The returned tibble includes: package name, build node, build stage, package
version number, status of stage, when the run started and ended, the command
the builder ran, branch of bioconductor and relevant git commit information.
}
\examples{
get_build_report("2025-12-29", branch="RELEASE_3_22", builder="nebbiolo2")

}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{list_all_branches}
\alias{list_all_branches}
\title{Get list of all Bioconductor git branches}
\usage{
list_all_branches(infoTbl = NULL)
}
\arguments{
\item{infoTbl}{downloaded paraquet info table retrieved from
get_bbs_table('info') that includes git_branch information.}
}
\value{
character() vector listing all options for "branch"
}
\description{
Get list of allBioconductor git branches
}
\details{
Get list of all Bioconductor git branches
}
\examples{
list_all_branches()

}
\seealso{
get_bbs_table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetTables.R
\name{get_all_bbs_tables}
\alias{get_all_bbs_tables}
\title{Retrieve all available Bioconductor build system parquet files}
\usage{
get_all_bbs_tables(
  assign_to_global = FALSE,
  useLocal = TRUE,
  updateLocal = FALSE
)
}
\arguments{
\item{assign_to_global}{logical(1) indicating if downloaded tables should be
assigned to the current global environment for access. Default is FALSE and
only assigned to custom environment through call to get_bbs_table.}

\item{useLocal}{A logical indicating if files should be read/downloaded from
a local file or read remotely.}

\item{updateLocal}{A logical indicating if using locally cached files, if the
files should be re-downloaded/updated. Theorically build reports are updated
daily so the local data may be out of date if not updated.}
}
\value{
invisible. tables are cached to custom environment unless
assign_to_global is TRUE.
}
\description{
Retrieve all available parquet tables at once.
}
\details{
This function is a quick wrapper around get_bbs_table to download
all available parquet files as tibbles.  By default no value is returned and
the tibbles are stored to the custom environment and can be easily accessed
through a call to get_bbs_table. If useLocal and updateLocal are TRUE, this
also becomes an easy helper to update/redownloaded all files to the locally
downloaded location.
}
\examples{
get_all_bbs_tables()

# Now that all are downloaded. Retrieve cached build_summary tibble
build_summary <- get_bbs_table("build_summary")

library(stringr)
library(dplyr)
# find the times the package BiocFileCache failed on linux nebbiolo builders
\donttest{
build_summary |> 
  filter(
    package == "BiocFileCache", 
    status \%in\% c("TIMEOUT", "ERROR"), 
    str_starts(node, "nebbiolo")
  ) 
}
}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{get_package_build_results}
\alias{get_package_build_results}
\title{Get the most recent build result data for a given package on a given branch}
\usage{
get_package_build_results(packagename, branch = "devel")
}
\arguments{
\item{packagename}{a single valid Bioconductor package name}

\item{branch}{a bioconductor branch (eg. 'devel', 'RELEASE_3_22')}
}
\value{
tibble of build status information or NULL if package or branch is not found
}
\description{
Get the build status information for a given package on a specified branch.
}
\details{
Utilizes data from the info and build_summary parquet files to
retrieve the latest build results for a specified package on a given
branch. If no branch is given it defaults to use 'devel'. The data returned
includes the node (builder machine name), stage (install, build, check),
package version number, status of stage, date stage completed, and git
information for the build (git branch, git commit hash, git commit date).
}
\examples{
get_package_build_results("BiocFileCache")
get_package_build_results("BiocFileCache", branch="RELEASE_3_22")

}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{get_failing_packages}
\alias{get_failing_packages}
\title{Get list of failing packages for the most recent build}
\usage{
get_failing_packages(branch = NULL, builder = NULL)
}
\arguments{
\item{branch}{a bioconductor branch (eg. 'devel', 'RELEASE_3_22')}

\item{builder}{name of a valid Bioconductor build machine (ie 'nebbiolo1')}
}
\value{
a tibble of failing packages
}
\description{
Get a list of failing (ERROR/TIMEOUT) packages for any given
branch and builder
}
\details{
Utilizes data from the build_summary and info parquet files to list
all packages failing (ERROR/TIMEOUT) in any stage of the process. Optionally
a Bioconductor git branch can be specified and by default will do both the
current devel and release. The report can be further filtered by optionally
specifying a build machine name.
The returned tibble includes: git branch, package name, package version
number, build machine node, collapsed column of stages and statuses.
}
\examples{
get_failing_packages("RELEASE_3_22")
get_failing_packages("RELEASE_3_22", "nebbiolo2")

}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{get_latest_branches}
\alias{get_latest_branches}
\title{Get currently active Bioconductor git branches}
\usage{
get_latest_branches(infoTbl = NULL)
}
\arguments{
\item{infoTbl}{downloaded paraquet info table retrieved from
get_bbs_table('info') that includes git_branch information.}
}
\value{
character(2) 'devel' and RELEASE_X_Y value eqivalent to current
release.
}
\description{
Get currently active Bioconductor git branches
}
\details{
Get currently active Bioconductor git branches
}
\examples{
get_latest_branches()

}
\seealso{
get_bbs_table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{package_failures_over_time}
\alias{package_failures_over_time}
\alias{package_failure_over_time}
\title{Get rough timeline overview of package failing events}
\usage{
package_failures_over_time(packagename, builder, failure_cluster_hours = 72)
}
\arguments{
\item{packagename}{a single valid Bioconductor package name}

\item{builder}{name of a valid Bioconductor build machine (ie 'nebbiolo1')}

\item{failure_cluster_hours}{numeric indicating the number of hours to treat
failures as a single event}
}
\value{
tibble of failure event information or NULL if package or builder is not found
}
\description{
Get rough timeline overview of a package failing events on a
specific builder
}
\details{
This function attempts to give an overview of how long a package has
been failing on a given builder. The time to treat failures as a single event
is distinguished by consecutive days and by the failure_cluster_hours
argument. Currently there is a different cadence of builds for release
compared to devel so this allows the flexibility to define the timeline.
Let's use the following example for interpretation of the result table:
```
  version episode       first_failure        last_failure n_failures
2   3.1.0       3 2025-12-15 02:57:57 2025-12-16 21:29:20          3
3   3.1.0       2 2025-12-13 02:32:33 2025-12-13 02:32:33          1
4   3.1.0       1 2025-12-08 21:30:14 2025-12-11 21:27:29          4
             stages statuses
2 buildsrc, checksrc    ERROR
3           checksrc    ERROR
4           buildsrc    ERROR
```
The package version being built on the build system for this package is
3.1.0. The episode refers to the number of failing episodic events. If we
look at the episode 1, the package started failing on December 8 and
continued to fail until December 11. During that time period there were 4
number of failures and they occurred during the build stage. The time between
the last failure of the first episode and the second episode is from December
11 to December 13; we can infer then that the package built cleanly on
December 12. During episode 2 the package failed solely on December 13 but
during the check stage. Episode 3 began on Dec 15 and lasted until December
16 and had 3 failures but they occurred in either the build or check stage.

This hopefully can help determine if package failures are intermittent and
how long a package is failing on a given builder.
}
\examples{
package_failures_over_time("BiocFileCache", "nebbiolo1", 24)

}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetTables.R
\name{get_bbs_table}
\alias{get_bbs_table}
\title{Retrieve a remotely read parquet file of Bioconductor build data}
\usage{
get_bbs_table(
  tblname = c("build_summary", "info", "propagation_status"),
  useLocal = TRUE,
  updateLocal = FALSE
)
}
\arguments{
\item{tblname}{A valid parquet file name. Currently available: build_summary,
info, or propagation_status.}

\item{useLocal}{A logical indicating if files should be read/downloaded
locally or read remotely.}

\item{updateLocal}{A logical indicating if using locally cached files, if the
files should be re-downloaded/updated. Theorically build reports are updated
daily so the local data may be out of date if not updated.}
}
\value{
returns a tibble.
}
\description{
Bioconductor Build results are at times cumbersome to work with and
interrogate. Parquet files of condensed reports are provided remotely. These
functions read the remote parquet files using the arrow package and return a
tibble. Once retrieved the table is stored in a custom environment for easy
quick reference either to use ad hocly or within pre-defined queries provided
by this package. If useLocal is chosen, the remote parquet files are
downloaded locally and cached using BiocFileCache. If already downloaded, the
local file is utilized unless updateLocal is also TRUE in which case the file
will be redownloaded to the local location before reading into the
session. If useLocal is FALSE, the files will be read remotely.
}
\details{
The get_bbs_table returns a tibble but is also saved in a custom
environment for quick easy subsequent reference. There are three tables
available and can be specified in get_bbs_table: build_summary, info, or
propagation_status. build_summary returns a tibble with the results of each
stage of the Bioconductor build process. info returns a tibble with
information regarding the package maintainer and git commit
status. propagation_status returns a tibble with data on if a package
propagated to the community.
}
\examples{
info <- get_bbs_table("info")
info

library(dplyr)
# all the package Lori Shepherd maintains
info |> filter(Maintainer == "Lori Shepherd") |> distinct(Package)


}
\seealso{
get_all_bbs_tables
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetTables.R
\docType{data}
\name{bbs_cache}
\alias{bbs_cache}
\alias{.bbs_cache}
\title{Custom Environment to store downloaded parquet tibbles.}
\format{
An environment
}
\usage{
.bbs_cache
}
\description{
A custom environment to store remotely read parquet files of
Bioconductor build reports. Once one of the three valid parquet files are
read in, the table is cached in this environment for quick easy reference
either to use ad hocly by the user or to be used within pre-defined queries
provided by this package.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BiocBuildReporter-package.R
\docType{package}
\name{BiocBuildReporter}
\alias{BiocBuildReporter-package}
\alias{BiocBuildReporter}
\title{BiocBuildReporter:  Functions to process a bioconductor build report database}
\description{
The BiocBuildReporter package provides access to years of Bioconductor build
system data, representing a comprehensive record of package builds across:
- **Thousands of packages** in the Bioconductor ecosystem
- **Multiple R versions** spanning several years of development
- **Multiple platforms** including Linux, macOS, and Windows
- **Different build stages** (install, build, check)
}
\details{
Please see vignette("BiocBuildReporter") for details on usage and pre-built
query functions.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/lshep/BiocBuildReporter.git}
  \item Report bugs at \url{https://github.com/lshep/BiocBuildReporter/issues}
}

}
\author{
\strong{Maintainer}: Lori Shepherd \email{lori.shepherd@roswellpark.org} (\href{https://orcid.org/0000-0002-3242-0582}{ORCID})

Authors:
\itemize{
  \item Sean Davis \email{seandavi@gmail.com}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{package_error_count}
\alias{package_error_count}
\title{Get a count of how many time a package failed on the Bioconductor
Build System}
\usage{
package_error_count(packagename, builder = NULL, branch = NULL)
}
\arguments{
\item{packagename}{a single valid Bioconductor package name}

\item{builder}{name of a valid Bioconductor build machine (ie 'nebbiolo1')}

\item{branch}{a bioconductor branch (eg. 'devel', 'RELEASE_3_22')}
}
\value{
tibble of data including counts of runs and errors or NULL if package
is not found
}
\description{
Get a count of how many time a package failed (ERROR/TIMEOUT) on the
Bioconductor Build System; optionally can specify a specific builder or
branch to filter results.
}
\details{
Utilizes data from the build_summary and info parquet files to
retrieve a count of errors (TIMEOUT/ERROR) for  a specified
package. Optionally a given builder machine name or Bioconductor branch may
be specified to filter request. If no builder is given it will return all. If
no branch is give it will also return all. The data returned
includes the node (builder machine name), package version number, stage
(install, build, check), total number of times that stage/builder tried to
run, total number of times the package failed during that stage, and the
given Bioconductor git branch.
NOTE: since 'devel' is on-going. There will be multiple 'devel' results
across multiple years not just the current devel. See example to filtering to
see most recent.
NOTE: If builder or branch is not found, function will continue and return
unfiltered results.
}
\examples{
package_error_count("BiocFileCache")
package_error_count("BiocFileCache", branch="RELEASE_3_22")
package_error_count("BiocFileCache", builder = "nebbiolo2", branch="RELEASE_3_22")

library(dplyr)
## devel will have more than just current devel
devErrors <- package_error_count("BiocFileCache", branch="devel")
## can use package version number to filter to current devel
devErrors |> filter(version == max(version))

}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{get_package_release_info}
\alias{get_package_release_info}
\title{Get git information for a given package across all Bioconductor releases}
\usage{
get_package_release_info(packagename)
}
\arguments{
\item{packagename}{a single valid Bioconductor package name}
}
\value{
tibble of release information or NULL if package is not found
}
\description{
Get the package version and git commit data for all releases in Bioconductor.
}
\details{
Utilizes data from the info parquet file to retrieve the equivalent
git information for every available Bioconductor release of a package. This
include the git commit hash and last commit date along with the package version.
}
\examples{
get_package_release_info("BiocFileCache")

}
\seealso{
get_bbs_table
}
\author{
Lori Shepherd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parquetQueries.R
\name{list_all_builders}
\alias{list_all_builders}
\title{Get list of all Bioconductor builders}
\usage{
list_all_builders(summaryTbl = NULL)
}
\arguments{
\item{summaryTbl}{downloaded paraquet build_summary table retrieved from
get_bbs_table('build_summary') that includes builder (node)  information.}
}
\value{
character() vector listing all options for "builder"
}
\description{
Get list of allBioconductor builders
}
\details{
Get list of all Bioconductor builders
}
\examples{
list_all_builders()

}
\seealso{
get_bbs_table
}
